// FreeFrameCV.cpp : Defines the entry point for the console application.

#include "stdafx.h"
#include <atlstr.h>

#include <cv.h>
#include <highgui.h>

#include "FreeFrame.h"
#include "FreeFramePlugin.h"

int _tmain(int argc, _TCHAR* argv[])
	{
	// check argument list
	if (argc != 2)
		{
		printf("Usage: FreeFrameCV aviFile\n");
		return 1;
		}

	// initialize the specified video file
	CvCapture* readVid = cvCreateFileCapture(argv[1]);
	IplImage* curFrame = cvQueryFrame(readVid);

	// initialize a FreeFrame plugin
	VideoInfoStruct vidInfo;
	vidInfo.BitDepth = FF_CAP_32BITVIDEO;	//FF_CAP_24BITVIDEO;
	vidInfo.FrameHeight = curFrame->height;
	vidInfo.FrameWidth = curFrame->width;
	vidInfo.Orientation = 1;

	FreeFramePlugin ffPlug;
	ffPlug.Load("resHeat.dll", vidInfo);

	// spit out some data about the loaded plugin
	CString plugName;
	ffPlug.GetName(plugName);
	printf("%s\n", (LPCTSTR)plugName);

	int numParams = ffPlug.GetNumParams();
	printf("parameters = %i\n", numParams);
	for (int i = 0; i < numParams; i++)
		{
		CString paramName;
		ffPlug.GetParamName(i, paramName);
		int paramType = ffPlug.GetParamType(i);
		float paramDefault = ffPlug.GetParamDefault(i);
		float paramValue = ffPlug.GetParamValue(i);
		printf("   %s %i %f %f\n", (LPCTSTR)paramName, paramType, paramDefault, paramValue);
		}

	// initialize a display window
	cvNamedWindow("display");
	int param0Val = 0;
	cvCreateTrackbar("param0", "display", &param0Val, 100, NULL);

	// cycle through all the video frames
	while (curFrame)
		{
		// convert the RGB image into an RGBA image
		IplImage* red = cvCreateImage(cvGetSize(curFrame), IPL_DEPTH_8U, 1);
		IplImage* green = cvCreateImage(cvGetSize(curFrame), IPL_DEPTH_8U, 1);
		IplImage* blue = cvCreateImage(cvGetSize(curFrame), IPL_DEPTH_8U, 1);
		cvSplit(curFrame, red, green, blue, NULL);

		IplImage* bitIm32 = cvCreateImage(cvGetSize(curFrame), IPL_DEPTH_8U, 4);
		IplImage* alpha = cvCreateImage(cvGetSize(curFrame), IPL_DEPTH_8U, 1);
		cvSet(alpha, CV_RGB(255,255,255));
		cvMerge(red, green, blue, alpha, bitIm32);
		bitIm32->origin = 1;

		// transform the frame using the FreeFrame plugin
		bool retCode = ffPlug.SetParamValue(0, param0Val / 100.0);
		float tmpVal = ffPlug.GetParamValue(0);
		printf("%f\n", tmpVal);
		ffPlug.ProcessFrame(bitIm32->imageData);

		// display the original and modified versions of the video frame
		cvShowImage("display", bitIm32);

		// go to the next frame
		curFrame = cvQueryFrame(readVid);
		if (cvWaitKey(1) >= 0)
			break;
		}

	// clean up
	cvDestroyWindow("display");
	cvReleaseCapture(&readVid);

	// initialize a video file to write to
//	double fps = cvGetCaptureProperty(readVid, CV_CAP_PROP_FPS);
//	CvVideoWriter* writeVid = cvCreateVideoWriter(argv[2], -1, fps,	cvGetSize(curFrame));

		// add the new frame to the output video
//		cvWriteFrame(writeVid, tracker.writeFrame);

	// clean up
//	cvReleaseVideoWriter(&writeVid);

	return 0;
	}
