#include "FreeFramePlugin.h"

FreeFramePlugin::FreeFramePlugin()
	{
	mainPtr = NULL;
	hInst = NULL;
	instID = NULL;
	}

FreeFramePlugin::~FreeFramePlugin()
	{
	Free();
	}

FreeFramePlugin::Free()
	{
	if (instID != NULL)
		{
		mainPtr(FF_DEINSTANTIATE, 0, (DWORD)instID);
		instID = NULL;
		}

	if (mainPtr != NULL)
		{
		mainPtr(FF_DEINITIALISE, 0, 0);
		mainPtr = NULL;
		}

	if (hInst != NULL)
		{
		FreeLibrary(hInst);
		hInst = NULL;
		}
	}

bool FreeFramePlugin::Load(const char* dllPath, VideoInfoStruct& videoInfo)
	{
	Free();

	hInst = LoadLibrary(dllPath);
	if (hInst == NULL)
		return false;

	mainPtr = (FF_Main_FuncPtr)GetProcAddress(hInst, "plugMain");
	if (mainPtr == NULL)
		return false;
	if (mainPtr(FF_INITIALISE, 0, 0).ivalue == FF_FAIL)
		return false;

	instID = mainPtr(FF_INSTANTIATE, (DWORD)&videoInfo, 0).instanceID;
	if (instID == LPVOID(FF_FAIL))
		return false;

	return true;
	}

bool FreeFramePlugin::GetInfo(PluginInfoStruct& pluginInfo)
	{
	PluginInfoStruct* pis = mainPtr(FF_GETINFO, 0, 0).PISvalue;
	if (pis == NULL)
		return false;
	pluginInfo = *pis;
	return true;
	}

bool FreeFramePlugin::GetExtendedInfo(PluginExtendedInfoStruct& pluginExtInfo)
	{
	PluginExtendedInfoStruct* peis = mainPtr(FF_GETEXTENDEDINFO, 0, 0).PEISvalue;
	if (peis == NULL)
		return false;
	pluginExtInfo = *peis;
	return true;
	}

bool FreeFramePlugin::GetName(CString& pluginName)
	{
	PluginInfoStruct* pis = mainPtr(FF_GETINFO, 0, 0).PISvalue;
	if (pis == NULL)
		return false;
	CopyFFString(pluginName, pis->PluginName, 16);
	return true;
	}

bool FreeFramePlugin::GetID(CString& pluginID)
	{
	PluginInfoStruct* pis = mainPtr(FF_GETINFO, 0, 0).PISvalue;
	if (pis == NULL)
		return false;
	CopyFFString(pluginID, pis->PluginUniqueID, 4);
	return true;
	}

int FreeFramePlugin::GetCaps(int capsIndex)
	{
	return (mainPtr(FF_GETPLUGINCAPS, capsIndex, 0).ivalue);
	}

int FreeFramePlugin::GetNumParams()
	{
	return (mainPtr(FF_GETNUMPARAMETERS, 0, 0).ivalue);
	}

bool FreeFramePlugin::GetParamName(int paramIndex, CString& paramName)
	{
	BYTE* pN = (BYTE*)mainPtr(FF_GETPARAMETERNAME, paramIndex, 0).svalue;
	if (pN == NULL)
		return false;
	CopyFFString(paramName, pN, 16);
	return true;
	}

int FreeFramePlugin::GetParamType(int paramIndex)
	{
	return (mainPtr(FF_GETPARAMETERTYPE, paramIndex, 0).ivalue);
	}

float FreeFramePlugin::GetParamDefault(int paramIndex)
	{
	return (mainPtr(FF_GETPARAMETERDEFAULT, paramIndex, 0).fvalue);
	}

float FreeFramePlugin::GetParamValue(int paramIndex)
	{
	return (mainPtr(FF_GETPARAMETER, paramIndex, (DWORD)instID).fvalue);
	}

bool FreeFramePlugin::SetParamValue(int paramIndex, float value)
	{
	SetParameterStruct sps;
	sps.index = paramIndex;
	sps.value = value;
	return (mainPtr(FF_SETPARAMETER, (DWORD)&sps, (DWORD)instID).ivalue == FF_SUCCESS);
	}

bool FreeFramePlugin::ProcessFrame(char* frame)
	{
	return (mainPtr(FF_PROCESSFRAME, (DWORD)frame, (DWORD)instID).ivalue == FF_SUCCESS);
	}

void FreeFramePlugin::CopyFFString(CString& dst, BYTE* src, int length)
	{
	// src should be fixed-length, but might be null-terminated
	char* buf = dst.GetBuffer(length + 1);
	memcpy(buf, src, length);
	buf[length] = 0;
	dst.ReleaseBuffer();
	dst.TrimRight();
	}
